#
# Copyright 2013 Infinidat Ltd. All rights reserved.
# Use is subject to license terms.
#
from copy import deepcopy


class Container(object):

    def __init__(self, initial_data):
        self._data = deepcopy(initial_data)

    def _split_indexed_key(self, token):
        return token[:token.index('[')], int(token[token.index('[') + 1: -1])

    def _walk(self, current, path, autoset_empty=False):
        items = self._split_path(path)
        while items:
            token = items.pop(0)
            if '[' in token:
                key, index = self._split_indexed_key(token)
                current = current[key]
                if isinstance(current, list):
                    if len(current) < index:
                        raise KeyError("list index out of range: {}".format(token))
                    current = current[index - 1]
                else:
                    raise KeyError()
            else:
                if autoset_empty and token not in current:
                    current[token] = {}
                current = current[token]
        return current

    def _split_path(self, path):
        return [item for item in path.split('.') if len(item)]

    def get(self, path=None):
        path = path or ''
        return self._walk(self._data, path)

    def set(self, path, value):
        # if parent is a dict, just update the dict
        # if parent is a list, replace the object in the value with a value
        # if parent does not exist, raise KeyError
        try:
            parent_path, key = path.rsplit('.', 1)
        except ValueError:
            parent_path, key = '', path

        parent = self._walk(self._data, parent_path, autoset_empty=True)
        if '[' in key:
            key, index = self._split_indexed_key(key)
            parent = parent.get(key)
            if len(parent) < index:
                raise KeyError("list index out of range: {}[{}]".format(key, index))
            parent.pop(index - 1)
            parent.insert(index - 1, value)
        else:
            parent.update({key: value})
