import ctypes
import infi.registry
import os
from infi.vendata.powertools.__version__ import __version__ as powertools_version

HANDLE = ctypes.c_void_p

LANG_ENGLISH = 9
MB_USERICON = 0x00000080L
LOAD_LIBRARY_AS_IMAGE_RESOURCE = 0x20


class MSGBOXPARAMS(ctypes.Structure):
    _fields_ = [("cbSize", ctypes.c_uint),
                ("hwndOwner", HANDLE),
                ("hInstance", HANDLE),
                ("lpszText", ctypes.c_char_p),
                ("lpszCaption", ctypes.c_char_p),
                ("dwStyle", ctypes.c_uint),
                ("lpszIcon", ctypes.c_char_p),
                ("dwContextHelpId", HANDLE),
                ("lpfnMsgBoxCallback", ctypes.c_void_p),
                ("dwLanguageId", ctypes.c_uint),
               ]


def PackMSGBOXPARAMS(icon_exe, icon_resource, caption, text):
    res = MSGBOXPARAMS()
    res.cbSize = ctypes.sizeof(res)
    res.hwndOwner = None
    res.hInstance = ctypes.windll.kernel32.LoadLibraryExA(str(icon_exe), 0, LOAD_LIBRARY_AS_IMAGE_RESOURCE)
    res.lpszText = text
    res.lpszCaption = caption
    res.dwStyle = MB_USERICON
    res.lpszIcon = icon_resource
    res.dwContextHelpId = None
    res.lpfnMsgBoxCallback = None
    res.dwLanguageId = LANG_ENGLISH
    return res


def get_icon_exe_path(product_name):
    registry = infi.registry.LocalComputer()
    products = registry.classes_root.get("Installer\\Products")
    for product_guid, product_hive in products:
        current_product_name = product_hive.values_store["ProductName"].to_python_object()
        if current_product_name == product_name:
            return product_hive.values_store["ProductIcon"].to_python_object().replace("1.ico", "2.exe")


def about():
    CAPTION = "About PowerTools"
    TEXT = "INFINIDAT Host PowerTools v." + powertools_version
    icon_path = get_icon_exe_path("Host PowerTools")
    if not icon_path or not os.path.exists(icon_path):
        ctypes.windll.user32.MessageBoxA(None, TEXT, CAPTION, 0)
    else:
        params = PackMSGBOXPARAMS(icon_path, 101, CAPTION, TEXT)
        ctypes.windll.user32.MessageBoxIndirectA(ctypes.byref(params))
