import cherrypy
import re
from logging import getLogger
from mako.lookup import TemplateLookup
from ... import TEMPLATES_PATH

logger = getLogger(__name__)

def is_supported_browser(request_headers):
    user_agent_string = request_headers.get("User-Agent", None)
    if user_agent_string is None:
        # we only block older versions of IE, we don't want to over-block users
        return True
    msie_matchobj = re.search("MSIE (\d+)\.(\d+)", user_agent_string)
    if msie_matchobj is None:
        # we only block older versions of IE - non-IE are ok
        return True
    msie_major_version = int(msie_matchobj.group(1))
    if msie_major_version >= 10:
        return True
    # this might still be MSIE 9 in compatibility mode, check the rendering engine version
    trident_matchobj = re.search("Trident/(\d+)\.(\d+)", user_agent_string)
    if trident_matchobj is None:
        return False
    trident_major_version = int(trident_matchobj.group(1))
    if trident_major_version >= 5:
        return True
    return False

def assert_browser():
    if not is_supported_browser(cherrypy.request.headers):
        logger.info("Unsupported browser. Redirecting")
        raise cherrypy.HTTPRedirect("/static/html/unsupported.html")

class AuthKey(object):
    authkey = None

    @classmethod
    def get_authkey(cls):
        import uuid
        if cls.authkey is None:
            cls.authkey = str(uuid.uuid4())
        return cls.authkey

    @classmethod
    def assert_authkey_tool(cls):
        request_authkey = cherrypy.request.params.get("auth_key", None)
        expected_authkey = cls.get_authkey()
        if expected_authkey != request_authkey:
            raise cherrypy.HTTPError(401, "Wrong authorization key")

def set_request_hook():
    cherrypy.tools.assert_browser = cherrypy.Tool('before_handler', assert_browser)
    cherrypy.tools.assert_authkey = cherrypy.Tool('before_handler', AuthKey.assert_authkey_tool)

class ViewBase(object):

    template_lookup = TemplateLookup(directories=[TEMPLATES_PATH], cache_enabled=False)

    def get_url_from_request(self):
        return "{}?{}".format(cherrypy.request.path_info, cherrypy.request.query_string)
