import cherrypy

from ......webapp import json_response

def refresh_storage_model():
    from infi.storagemodel import get_storage_model
    get_storage_model().rescan_and_wait_for()

def is_port_sees_the_system(port, system_serial):
    return any(remote_port.port_wwn for remote_port in port.discovered_ports if
               '5742b0f' in str(remote_port.port_wwn) and hex(system_serial)[2:] in str(remote_port.port_wwn))

def is_host_registered_in_system(system_serial):
    from infi.hbaapi import get_ports_collection
    from infi.vendata.powertools.utils.queries import get_system_object_by_serial
    hosts = set()
    our_ports = [port.port_wwn for port in get_ports_collection().get_ports()
                 if is_port_sees_the_system(port, system_serial)]
    if not our_ports:
        return False
    system = get_system_object_by_serial(system_serial, log_in=False)
    for wwn in our_ports:
        hosts.add(system.hosts.get_host_by_initiator_address(wwn))
    hosts.discard(None)
    return len(hosts) == 1  # TODO ports.difference(set(hosts[0].get_fc_ports()))

def register_host(system_serial):
    from infi.vendata.powertools.utils.queries import get_os_hostname
    from infi.vendata.powertools.system_register.engine import register, Abort
    import StringIO
    refresh_storage_model()
    namespace = {}
    namespace["--system"] = system_serial
    namespace["--hostname"] = get_os_hostname()
    namespace["--fix"] = False
    namespace["--auto-fix"] = False
    dummy_iostream = StringIO.StringIO()
    try:
        register(namespace, dummy_iostream)
    except Abort as error:  # pragma: no cover
        raise cherrypy.HTTPError(409, error.message)

class HostRegistration(object):
    @json_response
    def index(self, serial, **kwargs):
        method = getattr(self, cherrypy.request.method.upper())
        return method(int(serial))

    def POST(self, serial):
        register_host(serial)

    def GET(self, serial):
        return is_host_registered_in_system(serial)
