from ......webapp import json_response
from infi.pyutils.lazy import cached_method, clear_cache
from infi.vendata.powertools.volume_speedtest.engine import MultiprocessingPerformanceEngine

class Performance(object):
    @cached_method
    def device_serial_to_device_path(self, device_serial):
        device_serial = device_serial.replace(u'\ufffd', '\x00')
        from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices
        device = [device for device in get_infinidat_block_devices()
                  if device.get_scsi_serial_number() == device_serial]
        if len(device) == 0:
            raise Exception("Couldn't find device for serial {}".format(device_serial))
        return device[0].get_block_access_path()

    @cached_method
    def get_performance_engine(self, device_serial):
        device_path = self.device_serial_to_device_path(device_serial)
        return MultiprocessingPerformanceEngine(device_path, print_to_verbose_queue=True)

    @json_response
    def performance_test(self, first_request, device_serial, **kwargs):
        if first_request == "true":
            clear_cache(self)
        performance_engine = self.get_performance_engine(device_serial)
        if first_request == "true":
            import multiprocessing
            self.performance_test_process = multiprocessing.Process(target=performance_engine.run)
            self.performance_test_process.start()
        verbose_queue = performance_engine.get_verbose_queue()
        result = verbose_queue.get()
        return result

    @json_response
    def cancel_performance_test(self, device_serial, **kwargs):
        performance_engine = self.get_performance_engine(device_serial)
        performance_engine.cancel()
        self.performance_test_process.join()
