var view = {
    name: "PowerTools",        // This will be overridden by VMWare PowerTools

    select: function(selectorString, parent) {
        var elements = $(selectorString, parent);
        if(!elements.length) throw new Error(selectorString + ' Not found.');
        return elements;
    },

    getDataTableOptions: function () {
        // Sorting support for size columns
        jQuery.fn.dataTableExt.oSort['size-asc']  = function(x,y) {
            var nx = parseFloat(x) * (/TB$/.test(x) ? 1000 : 1);
            var ny = parseFloat(y) * (/TB$/.test(y) ? 1000 : 1);
            return ((nx < ny) ? -1 : ((nx > ny) ?  1 : 0));
        };
        jQuery.fn.dataTableExt.oSort['size-desc'] = function(x,y) {
            return -1 * jQuery.fn.dataTableExt.oSort['size-asc'](x, y);
        };
        // https://datatables.net/reference/option/
        var options = new Object();
        options.paging = false;
        options.ordering = true;
        options.processing = false;
        options.deferRender = true;
        options.info = false;
        options.searching = false;
        options.autoWidth = false;
        options.dom = "RJlfrtip";
        options.language = {loadingRecords: ''};
        return options;
    },

    createDataTable: function(selector, options) {
        var table = selector;
        table.on("preXhr.dt", function(e, settings, data) {
            spinnerFactory.blockUI("huge");
        });
        table.on("xhr.dt", function(e, settings, json) {
            $.unblockUI({fadeOut: 0});
            if (!json.success) {
                view.jsonError(json);
                return;
            }
            // copy json.return_value to json
            for (var key in json.return_value) {
                json[key] = json.return_value[key];
            }
        });
        $(document).on("click", ".ColVis_Restore", function(e) {
            $(".dataTable").dataTable().api().state.clear();
            window.location.reload();
        });
        table.dataTable(options);
        return table;
    },

    getLogMessage: function() {
        var collect_logs_path = "/plugin/views/host/collect_logs" + document.location.search;
        return 'Please <a target="_blank" href="' + collect_logs_path + '">download the logs</a> for troubleshooting.'
    },

    showErrorWithLogCollection: function(msg, reason) {
        var small_msg = view.getLogMessage();
        if (msg === null || msg === undefined) {
            msg = "An unexpected error has occurred.";
        }
        if (reason !== null && reason.length > 0) {
            small_msg = "Reason: " + reason + "<br><br>" + view.getLogMessage();
        }
        showError("ERROR", msg, small_msg);
    },

    jsonError: function(json, optional_msg) {
        if (json.unexpected_error) {
            view.showErrorWithLogCollection(optional_msg, json.error_message);
        } else {
            showError("ERROR", "ERROR: " + json.error_message);
        }
    },

    ajaxFail: function(response, status, xhr) {
        if (xhr.statusText == undefined && response.status == 0) {
            var msg = view.name + " server unreachable";
            showError("ERROR", msg);
        }
        else {
            var msg = xhr.statusText || xhr;
            view.showErrorWithLogCollection(null, msg);
        }
    },

    ajax: function(ajaxOptions, doneCallback, failCallback, finallyCallback) {

        _.defaults(ajaxOptions, {data: null, dataType: "json", cache: false});
        if (ajaxOptions.data && (ajaxOptions.type == 'POST' || ajaxOptions.type == 'PUT')) {
            ajaxOptions.data = JSON.stringify(ajaxOptions.data);
            ajaxOptions.contentType = 'application/json';
            ajaxOptions.processData = false;
        }

        var done = function(json) {
            if (!json.success) {
                if (!failCallback || !failCallback(json)) {
                    view.jsonError(json);
                }
            }
            else {
                if (doneCallback) doneCallback(json);
            }
            if (finallyCallback) finallyCallback(json);
        }

        var myFail = function(response, status, xhr) {
            if (!failCallback || !failCallback(null)) {
                view.ajaxFail(response, status, xhr);
            }
            if (finallyCallback) finallyCallback(null);
        };

        return $.ajax(ajaxOptions).done(done).fail(myFail);
    },

    spinAjax: function(ajaxOptions, doneCallback, failCallback, finallyCallback) {
        var spinFinallyCallback = function(json) {
            $.unblockUI({fadeOut: 0});
            if (finallyCallback) finallyCallback(json);
        };
        spinnerFactory.blockUI("huge");
        return view.ajax(ajaxOptions, doneCallback, failCallback, spinFinallyCallback);
    },

    showAbout: function() {
        var about = _.template($('#about_template').html())();
        $(about).dialog({
            modal: true,
            width: 540,
            height: 320,
            resizable: false,
            close: function(event, ui) {
                $(this).dialog("destroy");
            },
            buttons: {
                CLOSE: function(event) {
                    $(this).dialog("close");
                }
            }
        });
    },

    collect_logs: function(url) {
        $("#collect-logs-modal iframe").attr("src", url);
        $("#collect-logs-modal").dialog({
            modal: true,
            width: 540,
            height: 540,
            resizable: false,
            dialogClass: 'collect-logs-dialog',
            buttons: {
                CLOSE: function() {
                    $(this).dialog("close");
                }
            }
        });
    },

    show_summary_dialog: function(title, msgs, callback) {
        var div = $("<div/>")
        var table = $('<table id="summary-table" style="background: initial"></table>');
        function add_msg(msg) {
            var tr = $('<tr></tr>');
            var good_td = '<td><div class="action"><img src="/static/images/icon-good.png"></div></td>';
            table.append(tr);
            tr.append(good_td);
            tr.append('<td>' + msg + '</td>');
        }

        for (index = 0; index < msgs.length; index++) {
            add_msg(msgs[index]);
        }

        div.append(table);
        $(div).dialog({
            title: title,
            modal: true,
            width: 540,
            resizable: false,
            buttons: {
                OK: function() {
                    $(this).dialog("close");
                    if (callback) callback();
                }
            }
        });
    }
}


// Submit forms on ENTER
$(document).delegate('.ui-dialog', 'keydown', function(e) {
    var tagName = e.target.tagName.toLowerCase();
    if (tagName == 'input' && (e.target.type == 'button' || e.target.type == 'submit')) tagname ='button';
    if (e.which == $.ui.keyCode.ENTER && tagName != 'textarea' && tagName != 'select' && tagName != 'button' && tagName != 'a') {
        $(this).find('.ui-dialog-buttonset button').eq(0).trigger('click');
        return false;
    }
});


// Make underscore.js templates use {{ }} and {[ ]} to prevent clashing with Mako syntax
_.templateSettings = {
    evaluate:    /\{\[([\s\S]+?)\]\}/g,
    interpolate: /\{\{([\s\S]+?)\}\}/g
};


var tooltip = {
    show: function(obj, text) {
        $('<div/>')
            .attr('id', 'table-tooltip')
            .html(text)
            .appendTo('body')
            .position({
                my: "left+2 top-2",
                at: "left top",
                of: obj,
                collision: "flipfit",
                within: this.wrapper
            });
    },

    hide: function() {
        $('#table-tooltip').remove();
    },

    create: function(selector, text) {
        selector.on("mouseover", function(ev) { tooltip.show(ev, text); });
        selector.on("mouseout", function(ev) { tooltip.hide(); });
    }
}