from logging import getLogger
logger = getLogger(__name__)

class WebServer(object):
    _server = None

    @classmethod
    def start(cls, port, authkey_verification=True):
        """:returns: gevent.BaseServer instance"""
        from gevent.pywsgi import WSGIServer
        from ..webapp import WebApplication
        address, port = ("0.0.0.0", port)
        wsgi_app = WebApplication.get_wsgi_application(authkey_verification=authkey_verification)
        cls._server = WSGIServer((address, port), wsgi_app, log=None)
        cls._server.start()
        logger.info("Started WSGI server")

    @classmethod
    def stop(cls):
        logger.info("Stopped WSGI server")
        cls._server.close()
        cls._server.stop_accepting()
        cls._server.stop()

    @classmethod
    def get_address_and_port(cls):
        from socket import gethostname, gethostbyname_ex, getfqdn, gaierror
        _, port = cls._server.address
        try:
            _, _, addresses = gethostbyname_ex(gethostname())
        except gaierror:
            return '127.0.0.1', port
        addresses = [ad for ad in addresses if not (ad.startswith("169.254") or ad.startswith("127.0"))]
        address = addresses[0] if addresses else getfqdn()
        return address, port

    @classmethod
    def serve_forever(cls):
        return cls._server.serve_forever()
