from infi.vendata.powertools.utils.versioncheck import UnsupportedVersion
from ..system_register.engine import update_metadata, output_stream_context
from logging import getLogger
logger = getLogger(__name__)


def get_device_filesystem_name(device):
    from ..utils import safe_get_device_partitions
    partitions = safe_get_device_partitions(device)
    if len(partitions) != 1:
        return None
    return partitions[0].get_current_filesystem().get_name()

def volume_metadata_sync():
    from ..utils import get_mount_point
    from ..utils.queries import get_infinisdk_host_and_volume_by_device
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices
    from ..utils.versioncheck import is_supported
    from ..utils import auth
    from ..system_register.engine import Abort
    for device in get_infinidat_block_devices():
        serial = device.get_vendor().get_system_serial()
        if not is_supported(device.get_vendor().get_system_version()):
            continue
        if auth.CREDENTIALS_STORE.get_existing_credentials(serial):  # skip boxes that we can't log into
            try:
                _, volume = get_infinisdk_host_and_volume_by_device(device)

                mountpoint = get_mount_point(device)
                if mountpoint is None:  # filesystem not mounted
                    continue
                filesystem = get_device_filesystem_name(device)
                update_metadata(volume, mount_point=mountpoint, filesystem_type=filesystem)
            except (Abort, UnsupportedVersion):
                pass

def host_metadata_sync():
    from ..system_register.engine import update_metadata_on_controller, Abort
    from ..utils.queries import get_uniq_list_of_attached_boxes__by_serial
    from ..utils import auth
    from ..utils.versioncheck import is_supported
    for controller in get_uniq_list_of_attached_boxes__by_serial():
        serial = controller.get_vendor().get_system_serial()
        if not is_supported(controller.get_vendor().get_system_version()):
            continue
        if auth.CREDENTIALS_STORE.get_existing_credentials(serial):  # skip boxes that we can't log into
            try:
                update_metadata_on_controller(controller)
            except (Abort, UnsupportedVersion):
                pass

def metadata_sync():
    host_metadata_sync()
    volume_metadata_sync()

def silent_metadata_sync():
    from StringIO import StringIO
    with output_stream_context(StringIO()):
        try:
            metadata_sync()
        except:
            logger.exception("silent metadata sync failed")
