from ....case import FixableCase, Category
import os
from infi.pyutils.lazy import cached_method, clear_cache
from logging import getLogger
from infi.execute import execute, execute_assert_success
from pkg_resources import resource_filename
from infi.os_info.parse_version import parse_version


logger = getLogger(__name__)


ODM_FILEPATH = resource_filename(__name__, "infinidat.odm")
ODM_RPM_FILEPATH = resource_filename(__name__, "infinidat-odm.rpm")
MSG_FILEPATH = resource_filename(__name__, "NFINIDAT.msg")
CLEAN_ODM_FILEPATH = resource_filename(__name__, "infinidat_clean_odm")
CAT_PATH = "/etc/methods/NFINIDAT.cat"

class OdmAttributes(FixableCase):

    def _split_odm_attributes(self, odm_content):
        import re
        result = re.findall("(\w+):\s+(.*?)(?:\n\n|$)", odm_content.strip(), re.DOTALL)
        def split_value(value):
            return re.findall("\S+ = \S+", value)
        result = [(key, split_value(value)) for key, value in result]
        return result

    def run_fix(self):
        # We run 'infinidat_clean_odm' in case there are *some* attributes already in the ODM
        execute_assert_success(["rpm", "-U", ODM_RPM_FILEPATH])

    def run_test(self):
        cmd = execute(["rpm", "-q", "--queryformat", "%{RPMTAG_VERSION}", "infinidat-odm"])
        if 0 != cmd.get_returncode():
            self.fail("INFINIDAT ODM package is not installed")
        installed_version = cmd.get_stdout()
        our_version = execute_assert_success(["rpm", "-qp", "--queryformat", "%{RPMTAG_VERSION}", ODM_RPM_FILEPATH]).get_stdout()
        if parse_version(installed_version) < parse_version(our_version):
            self.fail("INFINIDAT ODM package is installed but not up to date")


    def get_description(self):
        return "Checking that INFINIDAT ODM package is installed"

    def get_kb_number(self):
        return "204014111"

class Catalog(FixableCase):
    def run_test(self):
        if not os.path.exists(CAT_PATH):
            self.fail("Catalog file does not exist")

        actual_content = execute_assert_success(["dspcat", "-g", CAT_PATH]).get_stdout()
        with open(MSG_FILEPATH, "rb") as fd:
            desired_content = fd.read()
        if actual_content != desired_content:
            self.fail("Catalog file is missing message entries")

    def run_fix(self):
        execute_assert_success(["gencat", CAT_PATH, MSG_FILEPATH])

    def get_description(self):
        return "Checking message catalog is installed"

    def get_kb_number(self):
        return "204074772"

def get_category():
    return Category("ODM", [OdmAttributes, Catalog])
