from ...case import Case, FixableCase, Category
from infi.execute import execute_assert_success
from gevent import sleep
EMPTY_TINYURL = "0000000"

# pylint: disable=R0904

class Pass(Case):
    def get_description(self):
        return "This test just passes"

    def run_test(self):
        pass

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "pass"

class Sleep(Case):
    def get_description(self):
        return "This test sleeps for two seconds"

    def run_test(self):
        sleep(2)

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "sleep"

class Execute(Case):
    def get_description(self):
        return "This test sleeps for two seconds with infi.execute"

    def run_test(self):
        execute_assert_success("sleep 3", shell=True)

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "execute"

class AssertTrue(Case):
    def get_description(self):
        return "This test assertTrue on False"

    def run_test(self):
        self.assertTrue(False, "We just assert on False")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "pass"

class Fail(Case):
    def get_description(self):
        return "This test is bound to fail"

    def run_test(self):
        self.fail("expect to fail")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "fail"

class Warn(Case):
    def get_description(self):
        return "This tests shows a warning"

    def run_test(self):
        self.warn("Beware")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "warn"

class Reboot(Case):
    def get_description(self):
        return "This requires a reboot"

    def run_test(self):
        self.ask_for_reboot("Because I said so")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "reboot"

class FixableFailure(FixableCase):
    def __init__(self, *args, **kwargs):
        super(FixableFailure, self).__init__(*args, **kwargs)
        self._fixed = getattr(self, "_fixed", False)

    def get_description(self):
        return "This test is a fixable failure"

    def run_test(self):
        if not self._fixed:
            self.fail("fix me please")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "fail"

    def get_fix_description(self):
        return """ marking as fixed.
        And nothing more. """

    def run_fix(self):
        self._fixed = True

class FixableFailureThatFailsFixing(FixableCase):
    def __init__(self, *args, **kwargs):
        super(FixableFailureThatFailsFixing, self).__init__(*args, **kwargs)
        self._fixed = getattr(self, "_fixed", False)

    def get_description(self):
        return "This test is a fixable failure that fails during the fix"

    def run_test(self):
        if not self._fixed:
            self.fail("fix me please")

    def get_kb_number(self):
        return EMPTY_TINYURL

    def get_name(self):
        return "fail"

    def get_fix_description(self):
        return """ Will through an excpetion intentionally during the fix """

    def run_fix(self):
        raise Exception("An error has occurred during the fix")


class Error(Case):
    def get_description(self):
        return "This test raises an unhandled error"

    def run_test(self):
        raise Exception("unhandled exception")

    def get_kb_number(self):
        return EMPTY_TINYURL

def get_category(pass_only=False):
    if pass_only:
        return Category("Demo", [Pass, Sleep, Execute, Warn, FixableFailure])
    return Category("Demo", [Pass, Sleep, Execute, Warn, Fail, Reboot, FixableFailure, FixableFailureThatFailsFixing, Error, AssertTrue])
