from infi import unittest
import mock


class TestCase(unittest.TestCase):
    def test_raise_skip_on_any_storage_related_error(self):
        from infi.vendata.powertools.settings.case import Skip
        from infi.storagemodel.errors import RescanIsNeeded
        from . import raise_skip_on_any_storage_related_error

        def func(*args, **kwargs):
            raise RescanIsNeeded()

        with self.assertRaises(Skip):
            raise_skip_on_any_storage_related_error(func)()
        with self.assertRaises(Skip):
            raise_skip_on_any_storage_related_error(func)(None, bar=None)

    # def test_get_number_of_fc_connections_to_each_infinibox(self):
    #     raise NotImplementedError()

    # def test_path_count(self):
    #     raise NotImplementedError()

    # def test_block_devices_with_reported_luns(self):
    #     raise NotImplementedError()


class BlockDevicesMatchReportedLuns_TestCase(unittest.TestCase):
    def test_get_actual__no_volumes_mapped(self):
        from infi.vendata.powertools.tests.mocks import three_system__registered
        from . import BlockDevicesMatchReportedLuns
        smock = three_system__registered.setUp()
        with smock:
            result = BlockDevicesMatchReportedLuns.get_actual_hctl_set_for_infinibox_block_devices()
            self.assertEquals(result, set())

    @unittest.parameters.iterate("map_volumes_to_lun0", [True, False])
    def test_get_actual__volumes_mapped(self, map_volumes_to_lun0):
        from infi.vendata.powertools.tests.mocks import five_systems_five_volumes_mapped_from_each
        from . import BlockDevicesMatchReportedLuns
        smock = five_systems_five_volumes_mapped_from_each.setUp(map_volumes_to_lun0)
        with smock:
            I, T, L, S = 2, 3, 5, 5
            result = BlockDevicesMatchReportedLuns.get_actual_hctl_set_for_infinibox_block_devices()
            self.assertEquals(len(result), I * T * L * S)

    @unittest.parameters.iterate("map_volumes_to_lun0", [True, False])
    def get_expected(self, map_volumes_to_lun0):
        from infi.vendata.powertools.tests.mocks import five_systems_five_volumes_mapped_from_each
        from . import BlockDevicesMatchReportedLuns
        smock = five_systems_five_volumes_mapped_from_each.setUp(map_volumes_to_lun0)
        with smock:
            I, T, L, S = 2, 3, 5, 5
            with mock.patch("infi.vendata.powertools.settings.cases.devices.get_reported_luns", new=lambda device: [0]):
                result = BlockDevicesMatchReportedLuns.get_expected_hctl_set_for_infinibox_block_devices()
            self.assertEquals(len(result), I * T * L * S if map_volumes_to_lun0 else 0)

    def test_case_test_method(self):
        from infi.vendata.powertools.tests.mocks import single_system_volume_mapped_to_lun0, single_system_three_mapped_volumes
        from . import BlockDevicesMatchReportedLuns
        smock = single_system_volume_mapped_to_lun0.setUp()
        with smock:
            with mock.patch("infi.vendata.powertools.settings.cases.devices.get_reported_luns", new=lambda device: [0]):
                BlockDevicesMatchReportedLuns(None).run_test()
        smock = single_system_three_mapped_volumes.setUp()
        with smock:
            with mock.patch("infi.vendata.powertools.settings.cases.devices.get_reported_luns", new=lambda device: [0, 1, 2, 3]):
                BlockDevicesMatchReportedLuns(None).run_test()


class PathCount_TestCase(unittest.TestCase):
    def test_get_number_of_fc_connections_to_each_infinibox(self):
        from infi.vendata.powertools.tests.mocks import three_system__unregistered
        from . import PathCount
        smock = three_system__unregistered.setUp(11110)  # TESTINF-3675
        with smock:
            result = PathCount.get_number_of_fc_connections_to_each_infinibox()
            self.assertEquals(result, {11112: 6, 11110: 6, 11111: 6})

    def test_case_test_method(self):
        from infi.vendata.powertools.tests.mocks import single_system_three_mapped_volumes
        from . import PathCount
        smock = single_system_three_mapped_volumes.setUp(11110)  # TESTINF-3675
        with smock:
            PathCount(None).run_test()
