from infi import unittest
from mock import patch
from munch import Munch
from .. import fc_driver_version
from infi.vendata.powertools.settings.case import Failure, Skip


RECOMMEDNED_OPTIONS = \
        [
        ("QLogic", "windows", "Windows Server 2012", "9.1.11.28"),
        ("Emulex", "redhat", "5", "8.03.01")
        ]

INCOMPATIBLE_OPTIONS = \
    [
        ("QLogic", "windows", "Windows Server 2012", "9.1.11.20"),
    ]

SUNOS_OPTIONS = \
    [
        ("QLogic", "sunos", "11", "1.2.3.4"),
    ]


class TestIncompatibleVersions(unittest.TestCase):
    @unittest.parameters.iterate("options", RECOMMEDNED_OPTIONS)
    def test_recommended_versions(self, options):
        manufacturer, platform, major, driver_version = options
        with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_major_version") as _get_major_version:
            with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_dist_name") as _get_dist_name:
                with patch.object(fc_driver_version, "get_ports_collection") as get_ports_collection:
                    _get_major_version.return_value = major
                    _get_dist_name.return_value = platform
                    port = Munch(model='', model_description='',
                                 manufacturer=manufacturer, driver_version=driver_version,
                                 port_symbolic_name='')
                    get_ports_collection().get_ports.return_value = [port]
                    case = fc_driver_version.SupportedFiberChannelDrivers(None)
                    case.run_test()

    @unittest.parameters.iterate("options", INCOMPATIBLE_OPTIONS)
    def test_incompatible_versions(self, options):
        manufacturer, platform, major, driver_version = options
        with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_major_version") as _get_major_version:
            with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_dist_name") as _get_dist_name:
                with patch.object(fc_driver_version, "get_ports_collection") as get_ports_collection:
                    _get_major_version.return_value = major
                    _get_dist_name.return_value = platform
                    port = Munch(model='', model_description='', manufacturer=manufacturer, port_symbolic_name='', driver_version=driver_version)
                    get_ports_collection().get_ports.return_value = [port]
                    case = fc_driver_version.SupportedFiberChannelDrivers(None)
                    with self.assertRaises(Failure):
                        case.run_test()

    @unittest.parameters.iterate("options", SUNOS_OPTIONS)
    def test_skip_version_check(self, options):
        manufacturer, platform, major, driver_version = options
        with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_major_version") as _get_major_version:
            with patch.object(fc_driver_version.SupportedFiberChannelDrivers, "_get_dist_name") as _get_dist_name:
                with patch.object(fc_driver_version, "get_ports_collection") as get_ports_collection:
                    _get_major_version.return_value = major
                    _get_dist_name.return_value = platform
                    port = Munch(model='', model_description='', manufacturer=manufacturer, port_symbolic_name='', driver_version=driver_version)
                    get_ports_collection().get_ports.return_value = [port]
                    case = fc_driver_version.SupportedFiberChannelDrivers(None)
                    with self.assertRaises(Skip):
                        case.run_test()

class TestCompanyIdentification(unittest.TestCase):

    def test_qlogic(self):
        ports = [
            Munch(model='dummy', model_description='dummy', manufacturer='QLogic', port_symbolic_name=''),
            Munch(model='QMH4062', model_description='QLogic QMH4062 iSCSI adapter', manufacturer=None, port_symbolic_name=''),
            Munch(model='HPAJ764A', model_description='HP 8Gb Dual Channel PCI-e 2.0 FC HBA', manufacturer=None, port_symbolic_name=''),
            Munch(model='QLE2562', model_description='PCI-Express Dual Channel 8Gb Fibre Channel HBA', manufacturer=None, port_symbolic_name='')
        ]
        for port in ports:
            self.assertEquals(fc_driver_version.get_fc_hba_company_from_port(port), 'QLogic')

    def test_emulex(self):
        ports = [
            Munch(model='dummy', model_description='dummy', manufacturer='Emulex', port_symbolic_name=''),
            Munch(model='LPe16004', model_description='Emulex LightPulse LPe16004', manufacturer=None, port_symbolic_name=''),
            Munch(model="AJ763A/AH403A", model_description="HP 8Gb Dual Channel PCI-e 2.0 FC HBA", manufacturer=None, port_symbolic_name="Emulex AJ763A/AH403A FV2.01A12 DV10.2.8020.1")
        ]
        for port in ports:
            self.assertEquals(fc_driver_version.get_fc_hba_company_from_port(port), 'Emulex')
