from mock import patch
from capacity import GB
from infi import unittest
from infi.vendata.smock import HostMock
from infi.vendata.powertools.settings.case import Skip, Failure
from infi.vendata.powertools.settings.cases.linux.base_cases import boot_from_san


def test_empty_smock():
    mock = HostMock()
    inventory = mock.get_inventory()  # noqa
    with mock:
        assert boot_from_san.is_booting_from_san() is False


def test_empty_local_disk():
    mock = HostMock()
    inventory = mock.get_inventory()
    inventory.add_local_disk(10*GB)
    with mock:
        assert boot_from_san.is_booting_from_san() is False


def create_partition_on_device_and_mount_it(device, mount_point='/'):
    from infi.storagemodel import get_storage_model
    disk = device.get_disk_drive()
    filesystem = get_storage_model().get_mount_manager().get_available_file_systems()[0]
    partition = disk.create_guid_partition_table().create_partition_for_whole_table(filesystem)
    filesystem.mount(partition.get_block_access_path(), '/')


def test_local_disk_mounted_to_slash():
    from infi.storagemodel import get_storage_model
    mock = HostMock()
    inventory = mock.get_inventory()
    inventory.add_local_disk(10*GB)
    with mock:
        [device] = get_storage_model().get_scsi().get_all_scsi_block_devices()
        create_partition_on_device_and_mount_it(device)
        assert boot_from_san.is_booting_from_san() is False


def test_infinibox_volume_mounted_to_slash():
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices
    mock = HostMock()
    inventory = mock.get_inventory()
    inventory.add_initiator()
    system = inventory.add_infinibox()
    inventory.zone_with_system__best_practices(system)
    inventory.register_host_in_system(system)
    pool = system.pools.create()
    volume = system.volumes.create(pool=pool)
    [host] = system.hosts.get_all()
    host.map_volume(volume)
    with mock:
        [device] = get_infinidat_block_devices()
        create_partition_on_device_and_mount_it(device)
        assert boot_from_san.is_booting_from_san() is True


class GenerateInitScriptTestCase(unittest.TestCase):
    def test_ubuntu(self):
        case = boot_from_san.GenerateInitScript(None)

        with patch.object(boot_from_san, "is_regeneration_of_initrd_needed") as is_regeneration_of_initrd_needed:
            is_regeneration_of_initrd_needed.return_value = False
            with self.assertRaises(Skip):
                case.run_test()

            is_regeneration_of_initrd_needed.return_value = True
            with patch.object(boot_from_san, "is_ubuntu") as is_ubuntu:
                is_ubuntu.return_value = True
                with self.assertRaises(Failure):
                    case.run_test()

    def test_not_ubuntu(self):
        case = boot_from_san.GenerateInitScript(None)

        with patch.object(boot_from_san, "is_regeneration_of_initrd_needed") as is_regeneration_of_initrd_needed:
            is_regeneration_of_initrd_needed.return_value = True

            with patch.object(boot_from_san, "is_ubuntu") as is_ubuntu:
                is_ubuntu.return_value = False

                with patch.object(boot_from_san, 'is_booting_from_san') as is_booting_from_san:
                    is_booting_from_san.return_value = False

                    with self.assertRaises(Skip):
                        case.run_test()

                    is_booting_from_san.return_value = True
                    with self.assertRaises(Failure):
                        case.run_test()
