
from infi.vendata.powertools.settings.cases.linux.base_cases import SoftwareDependency, SoftwareVersionDependency, ServiceAutoLoadTask, ServiceStartedTask
from infi.vendata.powertools.settings.cases.linux.base_cases import KernelModuleLoaded
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigRecommened, MultipathConfigDefaults
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigLocalExclusions, MultipathConfigInfinidatExclusions
from infi.vendata.powertools.settings.cases.linux.base_cases import FiberChannelDriverProperties, GenerateInitScript, SystemdUnitFile
from infi.vendata.powertools.settings.cases.linux.base_cases import IO_Scheduler_Rule, IO_Scheduler_AffectedSettings
from infi.vendata.powertools.settings.cases import fc_driver_version, connectivity, devices
from infi.vendata.powertools.settings.case import Category
from infi.pkgmgr import SusePackageManager
from infi.sysv_service import SuseInitService


class SG3Utils(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(SG3Utils, self).__init__(category, "202404121", "sg3_utils", SusePackageManager)


class Parted(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(Parted, self).__init__(category, "202319272", "parted", SusePackageManager)


class MultipathToolsPackageForSuse11(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsPackageForSuse11, self).__init__(category, "202404021", "multipath-tools", SusePackageManager)


class MultipathToolsPackageForSuse12(SoftwareVersionDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsPackageForSuse12, self).__init__(category, "202404021", "multipath-tools", "0.5.0-40.1", SusePackageManager)


class MultipathDaemonAutoStart(ServiceAutoLoadTask):
    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonAutoStart, self).__init__(category, "202319202", "multipathd",
                                                      "multipathd", SuseInitService)

class MultipathBootAutoStart(ServiceAutoLoadTask):
    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathBootAutoStart, self).__init__(category, "202319302", "boot.multipath",
                                                     "boot.multipath", SuseInitService)

class UdevBootAutoStart(ServiceAutoLoadTask):
    required_for_provisioning = True

    def __init__(self, category):
        super(UdevBootAutoStart, self).__init__(category, "202319312", "boot.udev",
                                                "boot.udev", SuseInitService)

class MultipathDaemonStartNow(ServiceStartedTask):
    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonStartNow, self).__init__(category, "202404141", "multipathd",
                                                      "multipathd", SuseInitService)

class MultipathModule(KernelModuleLoaded):
    def __init__(self, category):
        from platform import linux_distribution
        _, version, _ = linux_distribution()
        super(MultipathModule, self).__init__(category, "202319212", "dm_multipath")


def get_categories():
    from infi.os_info import get_platform_string
    from infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.powerpath import PowerPath, PowerPathCase, PowerPathExclusions
    is_suse_12 = 'suse-12' in get_platform_string()
    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_cases([FiberChannelDriverProperties])
    if is_suse_12:
        fc_hba_category.add_cases([SystemdUnitFile])

    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_cases([FiberChannelDriverProperties])

    MultipathToolsPackage = MultipathToolsPackageForSuse12 if is_suse_12 else MultipathToolsPackageForSuse11
    multipath_category = Category("Multipath", [])
    if PowerPath().should_fail():
        multipath_category.add_cases([PowerPathCase])
    else:
        if PowerPath().should_warn():
            multipath_category.add_cases([PowerPathCase, PowerPathExclusions])
        multipath_category.add_cases([MultipathToolsPackage, MultipathDaemonAutoStart, MultipathModule,
                                      MultipathDaemonStartNow, MultipathConfigDefaults,
                                      MultipathConfigRecommened, MultipathConfigInfinidatExclusions,
                                      MultipathConfigLocalExclusions])

    return [Category("SCSI", [SG3Utils, Parted]),
            multipath_category,
            Category("Performance", ([IO_Scheduler_Rule, IO_Scheduler_AffectedSettings])),
            fc_hba_category,
            Category("Boot From SAN", [GenerateInitScript]),
            devices.get_category(),
            connectivity.get_category()]
