
from infi.vendata.powertools.settings.cases.linux.base_cases import SoftwareDependency, ServiceAutoLoadTask, ServiceStartedTask
from infi.vendata.powertools.settings.cases.linux.base_cases import KernelModuleLoaded
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigRecommened, MultipathConfigDefaults
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigLocalExclusions, MultipathConfigInfinidatExclusions
from infi.vendata.powertools.settings.cases.linux.base_cases import FiberChannelDriverProperties, GenerateInitScript
from infi.vendata.powertools.settings.cases.linux.base_cases import IO_Scheduler_Rule, IO_Scheduler_AffectedSettings
from infi.vendata.powertools.settings.cases import fc_driver_version, connectivity, devices
from infi.vendata.powertools.settings.case import Category
from infi.pkgmgr import UbuntuPackageManager
from infi.sysv_service import UbuntuInitService

class SG3Utils(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SG3Utils, self).__init__(category, "202404121", "sg3-utils", UbuntuPackageManager)

class Parted(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(Parted, self).__init__(category, "202319272", "parted", UbuntuPackageManager)

class SCSITools(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SCSITools, self).__init__(category, "202404131", "scsitools", UbuntuPackageManager)

class MultipathToolsPackage(SoftwareDependency):

    required_for_provisioning = True
    PACKAGE_NAME = "multipath-tools"
    VERSION_TO_INSTALL = "0.4.9-3ubuntu7"
    BROKEN_VERSION = "0.4.9-3ubuntu7.5"

    def __init__(self, category):
        super(MultipathToolsPackage, self).__init__(category, "202404021", "multipath-tools", UbuntuPackageManager, self.VERSION_TO_INSTALL)

    def run_test(self):
        super(MultipathToolsPackage, self).run_test()

        # HPT-1680 - TODO when package is back to normal change the following code
        from infi.os_info.parse_version import parse_version
        installed_version = self._package_manager.get_installed_version(self.PACKAGE_NAME)['version']
        if parse_version(installed_version) >= parse_version(self.BROKEN_VERSION):
            msg = "{} package is installed but with a broken version: {}".format(self.PACKAGE_NAME, installed_version)
            self.warn(msg)

class MultipathToolsBootPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsBootPackage, self).__init__(category, "202404031", "multipath-tools-boot", UbuntuPackageManager, MultipathToolsPackage.VERSION_TO_INSTALL)

# This is needed to run before MultipathToolsBootPackage only because of the specific version hack
class KpartxPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        # TODO KB article
        super(KpartxPackage, self).__init__(category, "202404021", "kpartx", UbuntuPackageManager, MultipathToolsPackage.VERSION_TO_INSTALL)

# Ditto
class KpartxBootPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        # TODO KB article
        super(KpartxBootPackage, self).__init__(category, "202404021", "kpartx-boot", UbuntuPackageManager, MultipathToolsPackage.VERSION_TO_INSTALL)

class MultipathDaemonAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonAutoStart, self).__init__(category, "202319202", "multipath-tools",
                                                      "multipathd", UbuntuInitService)
class MultipathDaemonStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonStartNow, self).__init__(category, "202404141", "multipath-tools",
                                                      "multipathd", UbuntuInitService)

class MultipathModule(KernelModuleLoaded):
    def __init__(self, category):
        super(MultipathModule, self).__init__(category, "202319212", "multipath")

class MultipathdUdevMonitorPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorPackage, self).__init__(category, "202404181",
                                                           "multipathd-udev-monitor", UbuntuPackageManager)

class MultipathdUdevMonitorAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorAutoStart, self).__init__(category, "202404191", "multipathd-udev-monitor",
                                                      "multipathd_udev_monitor", UbuntuInitService)

class MultipathdUdevMonitorStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorStartNow, self).__init__(category, "202319292", "multipathd-udev-monitor",
                                                      "multipathd_udev_monitor", UbuntuInitService)

def get_categories():
    from infi.vendata.powertools.utils import get_distribution
    from infi.vendata.powertools.settings.cases.linux.base_cases.device_mapper.powerpath import PowerPath, PowerPathCase, PowerPathExclusions

    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_cases([FiberChannelDriverProperties])

    multipath_category = Category("Multipath", [])
    if PowerPath().should_fail():
        # in case of a coexistence issue we don't want to run the other tests
        multipath_category.add_cases([PowerPathCase])
    else:
        if PowerPath().should_warn():
            multipath_category.add_cases([PowerPathCase, PowerPathExclusions])
        # TODO the KpartxPackage, KpartxBootPackage and the specific versions in MultipathToolsPackage, MultipathToolsBootPackage
        # were only tested on 14.04. If we support a new ubuntu relase we might have to change them
        multipath_category.add_cases([KpartxPackage, KpartxBootPackage,
                                      MultipathToolsPackage, MultipathToolsBootPackage, MultipathDaemonAutoStart,
                                      MultipathModule, MultipathDaemonStartNow, MultipathConfigDefaults,
                                      MultipathConfigRecommened, MultipathConfigInfinidatExclusions,
                                      MultipathConfigLocalExclusions])

    categories = [Category("SCSI", [SG3Utils, SCSITools, Parted]),
            multipath_category,
            Category("Performance", ([IO_Scheduler_Rule, IO_Scheduler_AffectedSettings])),
            fc_hba_category,
            Category("Boot From SAN", [GenerateInitScript]),
            devices.get_category(),
            connectivity.get_category()]

    version_specific_categories = {
        "14.04": [Category("Multipathd Udev Monitor", [MultipathdUdevMonitorPackage,
                                                        MultipathdUdevMonitorAutoStart,
                                                        MultipathdUdevMonitorStartNow])]
    }

    categories += version_specific_categories.get(get_distribution().version, [])

    return categories
