from ....case import FixableCase, Category
from infi.execute import execute_assert_success
import re

# pylint: disable=R0904,W0201

SCSI_GENERIC_MODULE_NAME = "sgen"
SCSI_GENERIC_DEVICE_TYPE = "scsiclass,0c"

class ScsiGenericLoaded(FixableCase):
    def __init__(self, *args, **kwargs):
        super(ScsiGenericLoaded, self).__init__(*args, **kwargs)

    def _is_module_installed(self, module):
        modules_list = open("/etc/name_to_major", "rb").read()
        return re.search("({})\s+(\d*)\s*".format(module), modules_list, re.MULTILINE) is not None

    def _is_module_aliased(self, module, alias):
        modules_aliases = open("/etc/driver_aliases", "rb").read()
        return re.search("({})\s+(\"{}\")\s*".format(module, alias), modules_aliases, re.MULTILINE) is not None

    def run_test(self):
        msg = "SCSI generic not configured"
        self.assertTrue(self._is_module_installed(SCSI_GENERIC_MODULE_NAME), msg)
        self.assertTrue(self._is_module_aliased(SCSI_GENERIC_MODULE_NAME, SCSI_GENERIC_DEVICE_TYPE), msg)

    def _install_module(self, module, module_alias):
        execute_assert_success(["add_drv",
            "-v", "-i", "\"{}\"".format(module_alias), module])

    def _load_module(self, module, module_alias):
        execute_assert_success(["update_drv",
            "-v", "-a", "-i", "\"{}\"".format(module_alias), module])

    def get_fix_description(self):
        return "Configure SCSI generic module"

    def run_fix(self):
        if not self._is_module_installed(SCSI_GENERIC_MODULE_NAME):
            self._install_module(SCSI_GENERIC_MODULE_NAME, SCSI_GENERIC_DEVICE_TYPE)
        elif not self._is_module_aliased(SCSI_GENERIC_MODULE_NAME, SCSI_GENERIC_DEVICE_TYPE):
            self._load_module(SCSI_GENERIC_MODULE_NAME, SCSI_GENERIC_DEVICE_TYPE)
        execute_assert_success(["bootadm", "update-archive", "-v"])

    def get_description(self):
        return "Checking that SCSI generic is configured"

    def get_kb_number(self):
        return "202752942"


def get_category():
    return Category("Kernel Modules", [ScsiGenericLoaded])
