from ....case import FixableCase, Category
from infi.execute import execute_assert_success
import re

# pylint: disable=R0904,W0201

GLOBAL_CONFIG_FILENAME = '/etc/system'

class KernelTunables(FixableCase):
    def __init__(self, *args, **kwargs):
        super(KernelTunables, self).__init__(*args, **kwargs)

    def _get_case_config(self):
        from infi.vendata.powertools.conf import get_configuration
        return get_configuration().get('readiness.cases.solaris_kernel_tunables')

    def run_test(self):
        from re import match
        config = self._get_case_config()
        for tunable, expected in config.iteritems():
            actual = self._get_driver_tunable(tunable)
            if not actual:
                continue
            msg = "Tunable {} is {}, when it should be {}"
            equals = bool(match(actual, expected))
            self.assertTrue(equals, msg.format(tunable, actual, expected))

    def _is_driver_tunable_supported(self, tubable):
        return bool(self._get_driver_tunable(tubable))

    def _get_driver_tunable(self, param):
        data = execute_assert_success('echo "{} /U" | mdb -k'.format(param), shell=True).get_stdout()
        res = re.findall('{}:(\w+)\s*'.format(param), data, re.MULTILINE)
        return res[-1].strip('"') if res else None

    def _set_tunable_command(self, tunable, value):
        prefix = tunable[:tunable.index("_")]
        return "set {}: {} = {}\n".format(prefix, tunable, value)

    def _set_tunables(self):
        config = self._get_case_config()
        additional_config = "\n"
        for tunable, value in config.iteritems():
            if not self._is_driver_tunable_supported(tunable):
                continue
            additional_config += (self._set_tunable_command(tunable, value))
        open(GLOBAL_CONFIG_FILENAME, 'a').write(additional_config)

    def _backup_current_config(self):
        data = open(GLOBAL_CONFIG_FILENAME).read()
        open(GLOBAL_CONFIG_FILENAME + '.bak', 'w').write(data)

    def get_fix_description(self):
        return "Override scsi related kernel tunables"

    def run_fix(self):
        # TODO backup boot environment with beadm?
        self._backup_current_config()
        self._set_tunables()
        self.ask_for_reboot("Reboot is required for the changes to take effect")

    def get_description(self):
        return "Checking SCSI related kernel tunables"

    def get_kb_number(self):
        return "202403811"

def get_category():
    return Category("Kernel Tunables", [KernelTunables])
