from ....case import Case, Category
from infi.pkgmgr import SolarisPackageManager

class SoftwareDependency(Case):
    def __init__(self, category, kb_number, package_name):
        super(SoftwareDependency, self).__init__(category)
        self._kb_number = kb_number
        self._package_name = package_name

    def run_test(self):
        package_manager = SolarisPackageManager()
        self.assertTrue(package_manager.is_package_installed(self.get_package_name()),
                        "package {} is not installed".format(self.get_package_name()))

    def get_description(self):
        return "Checking that {} is installed".format(self.get_package_name())

    def get_kb_number(self):
        return self._kb_number

    def get_package_name(self):
        return self._package_name


class SunFCPrtSoftwareDependency(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(SunFCPrtSoftwareDependency, self).__init__(category, "202403801", "SUNWfcprt")


class SUNLuxopSoftwareDependency(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(SUNLuxopSoftwareDependency, self).__init__(category, "202403801", "SUNWluxop")


class SUNMpathCliSoftwareDependency(SoftwareDependency):
    required_for_provisioning = True

    def __init__(self, category):
        super(SUNMpathCliSoftwareDependency, self).__init__(category, "202403801", "SUNWmpathadm")


def get_category():
    return Category("Packages", [SunFCPrtSoftwareDependency, SUNLuxopSoftwareDependency, SUNMpathCliSoftwareDependency])
