#
# Copyright 2013 Infinidat Ltd. All rights reserved.
# Use is subject to license terms.
#
from infi.pyutils.contexts import contextmanager
import os
from gevent.fileobject import FileObject
from uuid import uuid4


@contextmanager
def gevent_async_open(filepath, mode='w'):
    with open(filepath, mode) as fd:
        fo = FileObject(fd, mode, close=False)  # Make file gevent-async (IZBOX-2726)
        # http://www.h-online.com/open/news/item/Ext4-data-loss-explanations-and-workarounds-740671.html
        yield fo
        if 'w' in mode or 'a' in mode:
            fo.flush()
            os.fsync(fd.fileno())


def windows_rename(src, dst):
    # From os.rename docs: on Windows, "there may be no way to implement an atomic rename when dst names an existing
    # file". so we try our best to at least not lose the data
    bak_path = dst + str(uuid4()) + ".bak"
    if os.path.exists(dst):
        os.rename(dst, bak_path)
    os.rename(src, dst)
    if os.path.exists(bak_path):
        os.remove(bak_path)


@contextmanager
def safe_write_file(filepath, mode='w', allow_makedirs=True):
    """Writes a file in a safe manner by first writing it to a tempfile, then moving it to the destination"""
    dirname = os.path.dirname(filepath)
    if not os.path.exists(dirname):
        if allow_makedirs:
            os.makedirs(dirname)
        else:
            raise IOError()
    tempfile_path = filepath + str(uuid4()) + '.tmp'
    with gevent_async_open(tempfile_path, mode) as fd:
        yield fd
    if os.name == "nt":
        windows_rename(tempfile_path, filepath)
    else:
        os.rename(tempfile_path, filepath)
