from infi.pyutils.contexts import contextmanager
from infi.gevent_utils.silent_greenlets import spawn, join
from gevent import sleep  # noqa - sleep imported by tests
from gevent.event import Event
from itertools import cycle
from sys import stderr


@contextmanager
def silence():
    try:
        yield
    except:
        pass


def main_loop(stop_event):
    cursor = cycle(r'/-\|')
    quit = False
    while not quit:
        with silence():
            stderr.write(cursor.next())
            stderr.flush()
        if stop_event.wait(0.5):
            quit = True
        with silence():
            stderr.write('\b')
    stderr.flush()


already_started = False
disabled = False


def disable():
    global disabled
    disabled = True


def enable():
    global disabled
    disabled = False


@contextmanager
def progress_bar_context(silent=False):
    global already_started
    global disabled
    if already_started or silent or disabled:
        yield
        raise StopIteration
    already_started = True
    stop_event = Event()
    greenlet = spawn(main_loop, stop_event)
    try:
        yield
    finally:
        stop_event.set()
        join(greenlet)
        already_started = False
