from infi.os_info.parse_version import parse_version
from infi.vendata.powertools.__version__ import __version__


FIRST_SUPPORTED_VERSION = parse_version('1.5')
FIRST_UNSUPPORTED_VERSION = parse_version('2.3')


class UnsupportedVersion(Exception):
    def __init__(self, version):
        msg = "Host PowerTools v{} does not support InfiniBox v{}".format(__version__, version)
        super(UnsupportedVersion, self).__init__(msg)


def is_supported(infinibox_version):
    v = parse_version(infinibox_version)
    return FIRST_SUPPORTED_VERSION <= v < FIRST_UNSUPPORTED_VERSION


def raise_if_unsupported(infinibox_version):
    if not is_supported(infinibox_version):
        raise UnsupportedVersion(infinibox_version)
