import os
from infi.vendata.powertools.utils import get_infinidat_multipath_block_devices

def map_existing_volume(system_serial, volume_id):
    from infi.vendata.powertools.volume_provision.engine import ProvisioningError
    from infi.storagemodel.vendor.infinidat.predicates import InfinidatVolumeExists
    from infi.vendata.powertools.utils import rescan_storagemodel
    from infi.vendata.powertools.utils.queries import get_system_object_and_host
    from infi.vendata.powertools.rescan.engine import clear_unit_attention_from_infinidat_storage_controllers
    system, host = get_system_object_and_host(system_serial)
    volume = system.volumes.get(id=volume_id)
    host.map_volume(volume)
    clear_unit_attention_from_infinidat_storage_controllers()
    rescan_storagemodel(InfinidatVolumeExists(system_serial, volume_id))

    def _is_provisioned_device(device):
        return device.get_vendor().get_system_serial() == system_serial and \
               device.get_vendor().get_volume_id() == volume_id
    if os.name == 'nt':
        devices = filter(_is_provisioned_device, get_infinidat_multipath_block_devices())
        if devices:
            disk_drive = devices[0].get_disk_drive()
            disk_drive.online() if not disk_drive.is_online() else None
        else:
            raise ProvisioningError("No matching MPIO device found for volume id {}".format(volume_id))


def get_volume_object(system_serial, volume):
    from infi.vendata.powertools.volume_provision.engine import ProvisioningError
    from infi.vendata.powertools.utils.queries import get_system_object_by_serial
    from infinisdk.core.exceptions import ObjectNotFound
    system = get_system_object_by_serial(system_serial)
    try:
        volume_id = int(volume)
        volume_obj = system.volumes.safe_get(id=volume_id)
        if volume_obj is not None:
            return volume_obj
    except ValueError:
        pass
    try:
        volume_obj = system.volumes.get(name=volume)
    except ObjectNotFound:
        raise ProvisioningError("Could not find volume {} on system {}".format(volume, system_serial))
    return volume_obj


def run(arguments):
    system_serial = int(arguments["--system"])
    volume = arguments["<volume>"]
    volume_obj = get_volume_object(system_serial, volume)
    map_existing_volume(system_serial, volume_obj.get_id())
    return volume_obj


def get_workout(arguments):
    system_serial = int(arguments["--system"])
    volume = arguments["<volume>"]
    volume_obj = get_volume_object(system_serial, volume)
    msg = "This command will map volume \"{}\" (id {}) to this host."
    return msg.format(volume_obj.get_name(), volume_obj.get_id())
